#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
test_geotext
----------------------------------

Tests for `geotext` module.
"""

import unittest
import geotext


class TestGeotext(unittest.TestCase):
    def setUp(self):
        pass

    def test_cities(self):

        text = """São Paulo é a capital do estado de São Paulo. As cidades de Barueri
                  e Carapicuíba fazem parte da Grade São Paulo. O Rio de Janeiro
                  continua lindo. No carnaval eu vou para Salvador. No reveillon eu 
                  quero ir para Santos."""
        result = geotext.GeoText(text).cities
        expected = [
            'São Paulo', 'São Paulo', 'Barueri', 'Carapicuíba', 'Rio de Janeiro', 'Salvador', 'Santos'
        ]
        self.assertEqual(result, expected)

        brazillians_northeast_capitals = """As capitais do nordeste brasileiro são:
                                            Salvador na Bahia, 
                                            Recife em Pernambuco, 
                                            Natal fica no Rio Grande do Norte, 
                                            João Pessoa fica na Paraíba, 
                                            Fortaleza fica no Ceará, 
                                            Teresina no Piauí, 
                                            Aracaju em Sergipe,
                                            Maceió em Alagoas e 
                                            São Luís no Maranhão."""
        result = geotext.GeoText(brazillians_northeast_capitals).cities
        # PS: 'Rio Grande' is not a northeast city, but is a brazilian city
        expected = [
            'Salvador', 'Recife', 'Natal', 'Rio Grande', 'João Pessoa', 'Fortaleza', 'Teresina', 'Aracaju', 'Maceió', 'São Luís'
        ]
        self.assertEqual(result, expected)


        brazillians_north_capitals = """As capitais dos estados do norte brasileiro são: 
                                        Manaus no Amazonas, 
                                        Palmas em Tocantins,
                                        Belém no Pará,
                                        Acre no Rio Branco."""
        result = geotext.GeoText(brazillians_north_capitals).cities
        expected = [
            'Manaus', 'Palmas', 'Belém', 'Rio Branco'
        ]
        self.assertEqual(result, expected)

        brazillians_southeast_capitals = """As capitais da região sudeste do Brasil são:
                                            Rio de Janeiro no Rio de Janeiro,
                                            São Paulo em São Paulo,
                                            Belo Horizonte em Minas Gerais,
                                            Vitória no Espírito Santo"""
        result = geotext.GeoText(brazillians_southeast_capitals).cities
        # 'Rio de Janeiro' and 'Sao Paulo' city and state name are the same, so appears 2 times, it's ok!
        expected = [
            'Rio de Janeiro', 'Rio de Janeiro', 'São Paulo', 'São Paulo', 'Belo Horizonte', 'Vitória'
        ]
        self.assertEqual(result, expected)

        brazillians_central_capitals = """As capitais da região centro-oeste do Brasil são: 
                                          Goiânia em Goiás, 
                                          Brasília no Distrito Federal,
                                          Campo Grande no Mato Grosso do Sul,
                                          Cuiabá no Mato Grosso."""
        result = geotext.GeoText(brazillians_central_capitals).cities
        expected = [
            'Goiânia', 'Goiás', 'Brasília', 'Campo Grande', 'Cuiabá'
        ]
        self.assertEqual(result, expected)

        brazillians_south_capitals = """As capitais da região sul são:
                                        Porto Alegre no Rio Grande do Sul,
                                        Floripa em Santa Catarina, 
                                        Curitiba no Paraná"""
        result = geotext.GeoText(brazillians_south_capitals).cities
        # PS: 'Rio Grande' is not a south city, but is a brazilian city
        expected = [
            'Porto Alegre', 'Rio Grande', 'Santa Catarina', 'Curitiba', 'Paraná'
        ]
        self.assertEqual(result, expected)

        result = geotext.GeoText('Rio de Janeiro y Havana', 'BR').cities
        expected = [
            'Rio de Janeiro'
        ]                
        self.assertEqual(result, expected)

    def test_nationalities(self):

        text = 'Japanese people like anime. French people often drink wine. Chinese people enjoy fireworks.'
        result = geotext.GeoText(text).nationalities
        expected = ['Japanese', 'French', 'Chinese']
        self.assertEqual(result, expected)

    def test_countries(self):

        text = """That was fertile ground for the emergence of various forms of
                  totalitarian governments such as Japan, Italy,
                  and Germany, as well as other countries"""
        result = geotext.GeoText(text).countries
        expected = ['Japan', 'Italy', 'Germany']
        self.assertEqual(result, expected)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from collections import namedtuple, Counter, OrderedDict
import re
import os
import io

_ROOT = os.path.abspath(os.path.dirname(__file__))


def get_data_path(path):
    return os.path.join(_ROOT, 'data', path)


def read_table(filename, usecols=(0, 1), sep='\t', comment='#', encoding='utf-8', skip=0):
    """Parse data files from the data directory

    Parameters
    ----------
    filename: string
        Full path to file

    usecols: list, default [0, 1]
        A list of two elements representing the columns to be parsed into a dictionary.
        The first element will be used as keys and the second as values. Defaults to
        the first two columns of `filename`.

    sep : string, default '\t'
        Field delimiter.

    comment : str, default '#'
        Indicates remainder of line should not be parsed. If found at the beginning of a line,
        the line will be ignored altogether. This parameter must be a single character.

    encoding : string, default 'utf-8'
        Encoding to use for UTF when reading/writing (ex. `utf-8`)

    skip: int, default 0
        Number of lines to skip at the beginning of the file

    Returns
    -------
    A dictionary with the same length as the number of lines in `filename`
    """

    with io.open(filename, 'r', encoding=encoding) as f:
        # skip initial lines
        for _ in range(skip):
            next(f)

        # filter comment lines
        lines = (line for line in f if not line.startswith(comment))

        d = dict()
        for line in lines:
            columns = line.split(sep)
            key = columns[usecols[0]].lower()
            value = columns[usecols[1]].rstrip('\n')
            d[key] = value
    return d


def build_index():
    """Load information from the data directory

    Returns
    -------
    A namedtuple with three fields: nationalities cities countries
    """

    nationalities = read_table(get_data_path('nationalities.txt'), sep=':')

    # parse http://download.geonames.org/export/dump/countryInfo.txt
    countries = read_table(
        get_data_path('countryInfo.txt'), usecols=[4, 0], skip=1)

    # parse http://download.geonames.org/export/dump/cities15000.zip
    cities = read_table(get_data_path('cities15000.txt'), usecols=[1, 8])

    # load and apply city patches
    city_patches = read_table(get_data_path('citypatches.txt'))
    cities.update(city_patches)

    Index = namedtuple('Index', 'nationalities cities countries')
    return Index(nationalities, cities, countries)


class GeoText(object):

    """Extract cities and countries from a text

    Examples
    --------

    >>> places = GeoText("London is a great city")
    >>> places.cities
    "London"

    >>> GeoText('New York, Texas, and also China').country_mentions
    OrderedDict([(u'US', 2), (u'CN', 1)])

    """

    index = build_index()

    def __init__(self, text, country=None):
        city_regex = r"[A-ZÀ-Ú]+[a-zà-ú]+[ \-]?(?:d[a-u].)?(?:[A-ZÀ-Ú]+[a-zà-ú]+)*"
        candidates = re.findall(city_regex, text)
        # Removing white spaces from candidates
        candidates = [candidate.strip() for candidate in candidates]
        self.countries = [each for each in candidates
                          if each.lower() in self.index.countries]
        self.cities = [each for each in candidates
                       if each.lower() in self.index.cities
                       # country names are not considered cities
                       and each.lower() not in self.index.countries]
        if country is not None:
            self.cities = [city for city in self.cities if self.index.cities[city.lower()] == country]

        self.nationalities = [each for each in candidates
                              if each.lower() in self.index.nationalities]

        # Calculate number of country mentions
        self.country_mentions = [self.index.countries[country.lower()]
                                 for country in self.countries]
        self.country_mentions.extend([self.index.cities[city.lower()]
                                      for city in self.cities])
        self.country_mentions.extend([self.index.nationalities[nationality.lower()]
                                      for nationality in self.nationalities])
        self.country_mentions = OrderedDict(
            Counter(self.country_mentions).most_common())

if __name__ == '__main__':
    print(GeoText('In a filing with the Hong Kong bourse, the Chinese cement producer said ...').countries)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

readme = open("README.rst").read()
history = open("HISTORY.rst").read().replace(".. :changelog:", "")

requirements = [
    # TODO: put package requirements here
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name="geotext",
    version="0.4.0",
    description="Geotext extracts countriy and city mentions from text",
    long_description=readme + "\n\n" + history,
    author="Yaser Martinez Palenzuela",
    author_email="yaser.martinez@gmail.com",
    url="https://github.com/elyase/geotext",
    packages=["geotext"],
    package_dir={"geotext": "geotext"},
    include_package_data=True,
    package_data={"geotext": ["geotext/data/*.txt"]},
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    keywords="geotext",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    test_suite="tests",
    tests_require=test_requirements,
)
